package com.iflytek.cloud;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.FaceRequestImpl;

/**
 * <h4>人脸识别类</h4>
 * <script>gShowExtraFace();</script>
 * <p>
 * 		人脸识别与{@link SpeakerVerifier 声纹密码}类似，作为人生理特征身份认证的其中一种，
 * 包括人脸检测、聚焦、注册、验证。在开始会话前，通过设置{@link SpeechConstant#WFR_SST}
 * 指定业务类型。人脸识别的结果说明，请参考<script>gShowUserGuideFile();</script>。
 * </p>
 * <p>
 * 		当前人脸识别支持以下图片格式：PNG，BMP, JPG，GIF。不需要针对不同格式设置格式参数，引擎
 * 根据传入的图片数据自动辨别。
 * </p>
 * <p>
 * 		人脸注册（reg），应用给终端用户指定一个{@link SpeechConstant#AUTH_ID 授权ID}后，上传
 * 用户的人脸照片，在服务器生成一个人脸模型，用于生理特征身份认证。
 * </p>
 * <p>
 * 		人脸验证（verify），应用指定由注册时的终端用户{@link SpeechConstant#AUTH_ID 授权ID}，上传
 * 用户的人脸照片，通过注册时在服务器已经保存的人脸模型，验证与注册时是否为同一人。
 * </p>
 * <p>
 * 		人脸检测（detect），检测出图片中人脸的位置 （支持多张人脸） ，返回位置的坐标。用于
 * 图片编辑、游戏等领域。
 * </p>
 * <p>
 * 		人脸聚焦（align），检测出图片中人脸的关键点坐标 （支持多张人脸），返回坐标集。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see SpeechUtility#createUtility
 * @see RequestListener
 * @see SpeechConstant
 */
public class FaceRequest extends SpeechInterface {

	private FaceRequestImpl mRequest;
	private Context mContext ;
	
	/**
	 * <h4>构造函数</h4>
	 * <p>
	 * 		创建实例用于人脸识别。
	 * </p>
	 * @param context
	 */
	public FaceRequest(Context context) {
		mContext = context;
	}

	/**
	 * <h4>开始人脸识别</h4>
	 * <p>
	 * 		开始人脸注册、验证、检测和聚焦等。在调用本函数开始开会前，可以通过
	 * {@link #setParameter(String, String)}来设置参数，指定业务类型等。
	 * </p>
	 * 
	 * @param img 图片数据
	 * @param listener 识别状态、结果监听器
	 * @return 错误码，0表示成功，否则出现错误，参考{@link ErrorCode}类的错误码定义。
	 * 
	 * @see #setParameter(String, String)
	 * @see #FaceRequest(Context)
	 * @see RequestListener
	 * @see #cancel()
	 */
	public int sendRequest(byte[] img,RequestListener listener) {
		synchronized (this) {
			if (mRequest != null) {
				mRequest.cancel();
				mRequest = null ;
			}
			mRequest =new FaceRequestImpl(mContext,mSessionParams);
			return mRequest.sendRequest(img,new InnerSpeechListener(listener));
		}
	}

	/**
	 * <h4>取消人脸识别</h4>
	 * <p>
	 * 		取消当前人脸识别会话。
	 * </p>
	 * 
	 * @see #sendRequest(byte[], RequestListener)
	 */
	public void cancel() {
		synchronized (this) {
			if(null != mRequest ) {
				mRequest.cancel();
				mRequest = null ;
			}
		}
	}


	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		在会话开始前，设置当前会话的业务类型等。
	 * </p>
	 * <p>
	 * 		与人脸相关的参数有：<br>
	 * {@link SpeechConstant#NET_TIMEOUT }： 网络连接超时时间<br>
	 * {@link SpeechConstant#WFR_SST }： 人脸识别业务类型<br>
	 * {@link SpeechConstant#AUTH_ID }： 授权ID<br>
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return true 设置成功，否则设置失败
	 * 
	 * @see #getParameter(String)
	 * @see #sendRequest(byte[], RequestListener)
	 */
	@Override
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}

	/**
	 * <h4>获取识别参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#NET_TIMEOUT}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * @param key 参数名称<br>
	 * @return 参数值
	 * 
	 * @see #setParameter(String, String)
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}

	public boolean destroy(){
		boolean ret = mRequest.destroy();
		if( ret ){
			ret = super.destroy();
		}
		
		return ret;
	}
	
	private static final int BASE_MSG_EVENT = 0;
	private static final int BASE_MSG_DATA = 1;
	private static final int BASE_MSG_END = 2;
	/**
	 * 通用消息回调接口，通过MainLooper转发到主线程
	 * @author yjzhao
	 *
	 */
	protected class InnerSpeechListener implements RequestListener {
		private RequestListener mOutListener = null;
		public InnerSpeechListener(RequestListener outlistener) {
			mOutListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				DebugLog.LogD("SpeechListener onMsg = " + msg.what);
				switch (msg.what) {
				case BASE_MSG_EVENT:
					mOutListener.onEvent(msg.arg1,(Bundle)msg.obj);
					break;
				case BASE_MSG_DATA:
					mOutListener.onBufferReceived((byte[])msg.obj);
					break;
				case BASE_MSG_END:
					mOutListener.onCompleted((SpeechError) msg.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onEvent(int eventType, Bundle params) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_EVENT,eventType,0,params);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onCompleted(SpeechError error) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_END, error);
			mUiHandler.sendMessage(msg);

		}

		@Override
		public void onBufferReceived(byte[] buffer) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_DATA,buffer);
			mUiHandler.sendMessage(msg);
		}
	}

}
