/*
 * 文件名：SpeechTranscripter.java 4.5.1049.1072 Sep 9th 2014
 * 描述 ：转写对象，包括开始转写、取消转写、停止转写等接口；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud;

import android.content.Context;

import com.iflytek.cloud.TranscripterListener;
import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.SpeechTranscripterImpl;
import com.iflytek.cloud.InitListener;
import com.iflytek.msc.MSC;

/**
 * <h3>语音转写类</h3>
 * <script>gShowExtraTranscripter();</script>
 * <p>
 * 		语音转写，与{@link SpeechRecognizer 语音听写}类似，是把音频转换为自然语言文本
 * 的功能；不一样的是，语音转写一次会话的音频时长，支持超过听写时
 * {@link SpeechConstant#KEY_SPEECH_TIMEOUT}限制的时间，目前最长可支持3小时（
 * 建议在2小时以内）。此外，因为长音频在上传时耗时更长，也更容易出现网络错误而中断
 * 当前上传的情况，所以，转写还支持断点续传，见{@link #startTranscripting}。
 * </p>
 * <p>
 * 		鉴于以上转写的特征，转写的音频，必须通过文件保存在本地，否则，在动态内存相对
 * 紧张的移动设备上，很可能在音频未上传完，而录音还在继续时，没有足够的内存而使会话
 * 失败；或无法进行断点续传。关于转写文件保存，见{@link SpeechConstant#IST_AUDIO_PATH}。
 * </p>
 * <p>
 * 		在转写长音频时，结果处理时间相对较长，视实际音频长短而定，为了使应用不需要长时间
 * 保持会话连接，当音频上传完（见{@link TranscripterListener#onEvent}）后，还可以先
 * 通过{@link #cancel}取消当次会话，在任意时侯，利用断点续传，查询转写的结果。见
 * {@link #startTranscripting}介绍。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createTranscripter}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getTranscripter}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createTranscripter}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see TranscripterListener
 * @see SpeechUtility#createUtility
 * @see SpeechUnderstander
 * @see SpeechSynthesizer
 * @see SpeechRecognizer
 */
public final class SpeechTranscripter extends SpeechInterface{
	
	// 单例对象
	private static SpeechTranscripter sInstance = null;
	
	// 客户端插件MSC实例
	private SpeechTranscripterImpl mTranscripter = null;
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getTranscripter}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 转写对象
	 * @see #destroy
	 * @see #getTranscripter
	 */
	public synchronized static SpeechTranscripter createTranscripter(Context context,InitListener listener) {
		synchronized( sSync ){
			if( sInstance==null && null!=SpeechUtility.getUtility() ){
				sInstance = new SpeechTranscripter(context,listener);
			}
		}//end of sync
		
		if(null != listener) {
			listener.onInit(ErrorCode.SUCCESS);
		}
		
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createTranscripter}创建单例对象。
	 * </p>
	 * 
	 * @return 转写对象
	 * 
	 * @see #createTranscripter
	 * @see #destroy
	 */
	public static SpeechTranscripter getTranscripter() {
		return sInstance;
	}
	
	/**
	 * 初始化转写对象。<br>
	 * @param context 上下文
	 */
	protected SpeechTranscripter(Context context,final InitListener listener) {
		super();
		// 分别创建speech.SpeechTranscripterImpl及com.iflytek.speech.SpeechTranscripterAidl
		if(MSC.isLoaded()){
			mTranscripter = new SpeechTranscripterImpl(context);
		}
	}

	/**
	 * <h4>开始转写</h4>
	 * <p>
	 * 		调用此函数，开始转写。在{@link SpeechConstant#AUDIO_SOURCE 录音模式}下，
	 * 保存在{@link SpeechConstant#IST_AUDIO_PATH}的音频文件将随着录音时间增长，
	 * 应注意存储空间是否够足，否则会出现音频数据保存失败而使会话中断。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话。一次会话，即从会话开始，到结
	 * 束为止。
	 * </p>
	 * <p>
	 * 		调用本函数开始转写后，通过SDK的录音机录取用户通过麦克风读入的音频（当
	 * {@link SpeechConstant#AUDIO_SOURCE} 值>=0时），或由应用层调用
	 * {@link #writeAudio}写入音频流，获取用于转写的音频。在完成音频
	 * 录入（包括麦克风或写音频流方式）后，通过调用{@link #stopTranscripting}告知SDK已完
	 * 成音频录入。在转写时，长音频时，不可避免的会有较长时间的静音，所以SDK自动关闭了
	 * VAD（Voice Activity Detection,静音抑制）。但应用层可以通过
	 * {@link SpeechConstant#KEY_SPEECH_TIMEOUT}设置录音的最长时间，参考
	 * {@link #setParameter}。
	 * </p>
	 * <p>
	 * 		在断点续传时，需要先通过{@link #setParameter}设置以下参数：<br>
	 * 会话ID：{@link SpeechConstant#IST_SESSION_ID}<br>
	 * 同步ID：{@link SpeechConstant#IST_SYNC_ID}<br>
	 * 已上传字节：{@link SpeechConstant#IST_AUDIO_UPLOADED}<br>
	 * 音频源：{@link SpeechConstant#AUDIO_SOURCE}<br>
	 * 并在调用此函数开始会话后，通过{@link #writeAudio}写入待续传的音频数据。
	 * 断点续传是音频已上传完成，只获取结果时，调用此函数开始会话后，直接调用
	 * {@link #stopTranscripting} 停止音频录入，等待结果返回。
	 * </p>
	 * <p>
	 * 		调用此函数开始会话前，可通过{@link #setParameter}设置会话参数。
	 * </p>
	 * <p>
	 * 		可通过{@link #cancel}取消当前的会话。
	 * </p>
	 * 
	 * @param listener 转写监听器，获取转写的状态和结果
	 * 
	 * @see #getTranscripter
	 * @see #stopTranscripting
	 * @see #setParameter
	 * @see #cancel
	 */
	public synchronized int startTranscripting(TranscripterListener listener)
	{
		DebugLog.LogD( "startTranscripting enter" );
		if (mTranscripter == null){
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		}
		
		if( this.isTranscripting() ){
			return ErrorCode.ERROR_ENGINE_BUSY;
		}
		
		mTranscripter.setParameter(SpeechConstant.PARAMS, null);
		mTranscripter.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
		
		final int ret = mTranscripter.startTranscripting(listener);
		clearOnceParameter();
		return ret;
	}
	
	/**
	 * 清理只应当前会话使用的参数
	 */
	private void clearOnceParameter(){
		this.setParameter( SpeechConstant.IST_SESSION_ID, null );
		this.setParameter( SpeechConstant.IST_AUDIO_UPLOADED, null );
		this.setParameter( SpeechConstant.IST_SYNC_ID, null );
		
	}
	
	
	/**
	 * <h4>写入音频数据</h4>
	 * <p>
	 * 		仅在音频源{@link SpeechConstant#AUDIO_SOURCE}为-1时，需要调用此接口
	 * 写入音频数据。
	 * </p>
	 * <p>
	 * 		在首次写入音频前，应该先调用{@link #startTranscripting}函数,以请求开始一次
	 * 会话，并等待 {@link SpeechEvent#EVENT_SESSION_BEGIN}事件消息到达后开始，
	 * 以避免由于会话未开始而造成写音频失败。
	 * </p>
	 * <p>
	 * 		在循环调用此函数写入音频时，应考虑网络带宽，进行必要的等待，以避免由于缓存
	 * 不够而造成错误。当前写入音频大小，应该小于等于 由
	 * {@link SpeechEvent#EVENT_IST_CACHE_LEFT}事件返回的音频大小。
	 * </p>
	 * <p>
	 * 		在写入音频数据时，服务器可能已返回结果，见{@link TranscripterListener#onResult}；
	 * 或在音频写完后，需要等待一段时间后才有结果返回，见
	 * {@link SpeechEvent#EVENT_IST_RESULT_TIME}。
	 * </p>
	 * <p>
	 * 		在服务器收到全部音频数据时，会在返回{@link SpeechEvent#EVENT_IST_UPLOAD_BYTES}
	 * 消息时，携带值为true的{@link SpeechEvent#KEY_EVENT_IST_UPLOAD_COMPLETE}
	 * 字段。
	 * </p>
	 * <p>
	 * 		在出错误时，会通过{@link TranscripterListener#onError}返回，说明本次会话已经
	 * 失败，应停止定入音频。可以通过再次调用{@link #startTranscripting}后，再调用此函
	 * 数，进行续传或从头开始上传音频。
	 * </p>
	 * 
	 * @param buffer 写入的录音数据<br>
	 * @param offset 录音数据起始位置<br>
	 * @param length 录音数据长度<br>
	 * @return 是否成功写入数据
	 * 
	 * @see SpeechConstant#AUDIO_SOURCE
	 * @see SpeechEvent#EVENT_IST_UPLOAD_BYTES
	 * @see SpeechEvent#EVENT_IST_CACHE_LEFT
	 * @see TranscripterListener
	 * @see #startTranscripting
	 * @see #stopTranscripting
	 * @see #cancel
	 */
	public boolean writeAudio(byte[] buffer,int offset,int length)
	{
		if( null != mTranscripter && mTranscripter.isTranscripting() ){
			return mTranscripter.writeAudio(buffer, offset, length);
		}else {
			DebugLog.LogE("SpeechTranscripter writeAudio failed, is not running");
			return false;
		}
	}
	
	/**
	 * <h4>停止录音</h4>
	 * <p>
	 * 		结束本次音频输入，通知服务器当次会话的音频输入已完成。
	 * </p>
	 * <p>
	 * 		停止录音后，如果当前转写还有结果没有返回，将会返回剩下的结果，结果返回时间
	 * 视网络和音频而定，且当前依然为正在转写的状态，直到返回完最后一个结果，或出现
	 * 错误。
	 * </p>
	 * 
	 * @see #startTranscripting
	 * @see #writeAudio
	 * @see #cancel
	 * @see TranscripterListener
	 */
	public void stopTranscripting()
	{
		DebugLog.LogD( "stopTranscripting enter" );
		if( null != mTranscripter && mTranscripter.isTranscripting() ) {
			mTranscripter.stopTranscripting();
		} else {
			DebugLog.LogE("SpeechTranscripter stopListening failed, is not running");
		}
	}
	
	/**
	 * <h4>是否正在转写</h4>
	 * <p>
	 * 		返回当前是否在转写中。从调用{@link #startTranscripting}开启新会话后，
	 * 至{@link TranscripterListener#onResult}返回最后一个结果，或
	 * {@link TranscripterListener#onError}返回错误，或通过{@link #cancel}取消
	 * 会话为止，为处于正在转写会话的状态。
	 * </p>
	 * 
	 * @see #startTranscripting
	 * @see #stopTranscripting
	 * @see #cancel
	 */
	public boolean isTranscripting()
	{
		if(null != mTranscripter && mTranscripter.isTranscripting()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * <h4>取消转写</h4>
	 * <p>
	 * 		取消当次转写会话。取消转写后，本次会话结束，没有返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #startTranscripting
	 * @see #stopTranscripting
	 */
	public void cancel() {
		DebugLog.LogD( "cancel enter" );
		if( null != mTranscripter && mTranscripter.isTranscripting() ) {
			mTranscripter.cancel(false);
		} else {
			DebugLog.LogE("SpeechTranscripter cancel failed, is not running");
		}
	}
	
	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		需要在调用{@link #startTranscripting}前，调用此函数设置参数，方可对当次会话生效。
	 * 可设置的参数有：
	 * <br>{@link SpeechConstant#NET_TIMEOUT}： 网络连接超时时间；
	 * <br>{@link SpeechConstant#KEY_SPEECH_TIMEOUT}：录音超时；
	 * <br>{@link SpeechConstant#KEY_REQUEST_FOCUS}：音频焦点；
	 * <br>{@link SpeechConstant#AUDIO_SOURCE}：音频源；
	 * <br>{@link SpeechConstant#SAMPLE_RATE}：识别采样率；
	 * <br>{@link SpeechConstant#ASR_PTT}：设置是否有标点符号；
	 * <br>{@link SpeechConstant#RESULT_TYPE}：识别结果类型；
	 * <br>{@link SpeechConstant#IST_AUDIO_PATH}：录音保存路径；
	 * <br>{@link SpeechConstant#IST_SESSION_ID}：会话ID，在续传时，必需；
	 * <br>{@link SpeechConstant#IST_AUDIO_UPLOADED}：已上传字节，在续传时，必需；
	 * <br>{@link SpeechConstant#IST_SYNC_ID}：同步ID，在续传时，必需；
	 * <br>{@link SpeechConstant#IST_SESSION_TRY}：会议重试次数；
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型（目前转写仅支持在线模式）；
	 * 
	 * <br>其中,以下与续传有关的参数仅对当次会话有效：
	 * <br>{@link SpeechConstant#IST_SESSION_ID}：会话ID；
	 * <br>{@link SpeechConstant#IST_AUDIO_UPLOADED}：已上传字节；
	 * <br>{@link SpeechConstant#IST_SYNC_ID}：同步ID；
	 * </p>
	 * <p>
	 * 		当前转写支持未压缩的16位，单声道，采样率为16000或8000，字节顺序
	 *  为Little-Endian的Windows PCM音频。
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return 错误码，0表示成功
	 * 
	 * @see SpeechConstant
	 * @see #getParameter
	 * @see #startTranscripting
	 */
	@Override
	public boolean setParameter( String key, String value ) {
		return super.setParameter( key, value );
	}
	
	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		以下参数只能在会话期间能获取到：
	 * <br>{@link SpeechConstant#IST_AUDIO_PATH}： 录音保存路径；
	 * <br>{@link SpeechConstant#IST_SESSION_ID}：会话ID；
	 * <br>{@link SpeechConstant#IST_AUDIO_UPLOADED}：已上传字节；
	 * <br>{@link SpeechConstant#IST_SYNC_ID}：同步ID；
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#SAMPLE_RATE}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key 参数名称,与{@link #setParameter}一致<br>
	 * @return 参数值
	 * 
	 * @see #setParameter
	 */
	public String getParameter( String key ) {
		if( null != this.mTranscripter ){
			if( SpeechConstant.IST_SESSION_ID.equalsIgnoreCase(key) ){
				return mTranscripter.getSessionID();
			}else if( SpeechConstant.IST_AUDIO_UPLOADED.equalsIgnoreCase(key) ){
				return String.valueOf( mTranscripter.getUploadBytes() );
			}else if( SpeechConstant.IST_SYNC_ID.equalsIgnoreCase(key) ){
				return mTranscripter.getSessionID();
			}else if( SpeechConstant.IST_AUDIO_PATH.equalsIgnoreCase(key) ){
				return mTranscripter.getAudioPath();
			}
		}

		return super.getParameter( key );
	}

	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createTranscripter}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isTranscripting}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createTranscripter}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createTranscripter
	 * @see #cancel
	 */
	public boolean destroy() {
		boolean ret = true;
		
		final SpeechTranscripterImpl impl = mTranscripter;
		if( impl != null ){
			ret = impl.destroy();
		}
		
		if( ret &&(ret=super.destroy()) ){
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
		}
		
		return ret;
	}
}
