package com.iflytek.cloud;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.RemoteException;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.TextUnderstanderImpl;
import com.iflytek.msc.MSC;
import com.iflytek.speech.TextUnderstanderAidl;

/**
 * <h3>文本语义类</h3>
 * <p>
 * 		通过此类，把自然语言的文本字符串，转换成以一定数据结构的文本数据，应用
 * 通过结构中的数据，获取用户的意图，进行下一步处理。
 * </p>
 * <p>
 * 		关于语义理解的介绍，请参考语音语义类{@link SpeechUnderstander}。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createTextUnderstander}
 * 创建一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。
 * 调用者可通过{@link #getTextUnderstander()}获取当前已经创建的单例。在销毁本类的单例
 * 对象后，需要先通过{@link #createTextUnderstander}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility#createUtility
 * @see SpeechUnderstander
 *
 */
public class TextUnderstander extends SpeechInterface{
	
	private TextUnderstanderImpl mUnderstander = null;
	
	private TextUnderstanderAidl mUnderstanderAidl = null;
	/**
	 *  识别单例对象
	 */
	private static TextUnderstander sInstance = null;
	/*
	 *  语记识别监听
	 */
	private InternalListener mInternalAidlListener = null;
	
	/**
	 * 初始化回调监听
	 */
	private InitListener mInitListener = null;
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getTextUnderstander()}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 文本语义对象
	 * @see #destroy
	 * @see #getTextUnderstander()
	 */
	public synchronized static TextUnderstander createTextUnderstander(Context context,InitListener listener) {
		synchronized( sSync ){
			if(sInstance==null && null!=SpeechUtility.getUtility() ){
				sInstance = new TextUnderstander(context,listener);
			}
		}//end of sync
		
		return sInstance;
	}

	/**
	 * 构造函数，不进行任何操作。<br>
	 */
	protected TextUnderstander(Context context,InitListener listener)
	{
		super();
		
		mInitListener = listener;
		
		// 分别创建speech.TextUnderstanderImpl及com.iflytek.speech.TextUnderstanderAidl
		if(MSC.isLoaded())
		{
			mUnderstander = new TextUnderstanderImpl(context);
		}
		
		//创建语记识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
			&& utility.getEngineMode() != ENGINE_MODE.MSC) 
		{
			mUnderstanderAidl = new TextUnderstanderAidl(context.getApplicationContext(), listener);
		}
		else if(null != listener) 
		{
			//放在主线程中回调，修复用户在回调中立即使用本对象造成的空指针问题
			Message.obtain(mUiHandler, ErrorCode.SUCCESS, 0, 0, null).sendToTarget();
		}
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createTextUnderstander}创建单例对象。
	 * </p>
	 * 
	 * @return 文本语义对象
	 * 
	 * @see #createTextUnderstander
	 * @see #destroy
	 */
	public static TextUnderstander getTextUnderstander() {
		return sInstance;
	}
	
	private Handler mUiHandler = new Handler(Looper.getMainLooper())
	{
		@Override
		public void handleMessage(Message msg) 
		{
			//listener 判断是否为null
			if(mInitListener == null)
				return;
			mInitListener.onInit(ErrorCode.SUCCESS);
		}
	};
	
	
	protected void reInit(Context context) 
	{	
		//创建语记识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
				&& utility.getEngineMode() != ENGINE_MODE.MSC) 
		{
			//判断服务是否可用，在用户升级语记造成的服务端开时会出现服务不可用的情况
			if(null != mUnderstanderAidl && !mUnderstanderAidl.isAvailable())
			{
				mUnderstanderAidl.destory();
				mUnderstanderAidl = null;
			}
			mUnderstanderAidl = new TextUnderstanderAidl(context.getApplicationContext(), mInitListener);
		} 
		else if(null != mInitListener) 
		{
			if(null != mUnderstanderAidl)
			{
				mUnderstanderAidl.destory();
				mUnderstanderAidl = null;
			}
		}
	}
	
	/**
	 * <h4>理解文本</h4>
	 * <p>
	 * 		调用此函数，传入自然语言的文本内容，获取语义理解的结果。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话。一次会话，即从会话开始，到结
	 * 束为止。
	 * </p>
	 * <p>
	 * 		在调用本函数开始会话前，通过{@link #setParameter(String, String)}设置相
	 * 应的参数。通过{@link #cancel()}取消本次会话。
	 * </p>
	 * 
	 * @param text 需要理解的文本
	 * @param listener 监听器
	 * 
	 * @see #setParameter(String, String)
	 * @see #cancel()
	 * @see #getTextUnderstander()
	 * @see TextUnderstanderListener
	 */
	public int understandText(String text, TextUnderstanderListener listener)
	{
		//语记已经安装，并且本地或者混合识别,需要走语记
		ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_NLU, mUnderstanderAidl);
		DebugLog.LogD("start engine mode = " + mode.toString());
		//@{ start:去除语记支持
		/*if(mode == ENGINE_MODE.PLUS)
		{
			//没有安装语记
			if(mUnderstanderAidl == null) 
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mUnderstanderAidl.setParameter(SpeechConstant.PARAMS, null);
			mUnderstanderAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());

			//主线程抛出回调消息
			mInternalAidlListener = new InternalListener(listener);				
			return mUnderstanderAidl.understandText(text, mInternalAidlListener.mAidlListener);
		} 
		else 
		{
			if(mUnderstander == null)
			{
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			}
			mUnderstander.setParameter(mSessionParams);
			return mUnderstander.understandText(text, listener);
		}*/
		//@ end:去除语记支持}
		if(mUnderstander == null)
		{
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		}
		mUnderstander.setParameter(mSessionParams);
		return mUnderstander.understandText(text, listener);
	}

	/**
	 * <h4>是否在会话中</h4>
	 * <p>
	 * 		通过此函数，获取当前SDK是否正在进行会话。应用层可通过此函数，查询能否
	 * 开始一路新的会话等。
	 * </p>
	 * @return 会话状态，true：正在会话中；false：不在会话中。
	 * 
	 * @see #understandText(String, TextUnderstanderListener)
	 * @see #cancel()
	 */
	public boolean isUnderstanding()
	{
		if(null != mUnderstander && mUnderstander.isUnderstanding()) 
		{
			return true;
		}
		else if(null != mUnderstanderAidl && mUnderstanderAidl.isUnderstanding()) 
		{
			return true;
		} 
		else 
		{
			return false;
		}
	}
	
	/**
	 * <h4>取消会话</h4>
	 * <p>
	 * 		通过此函数取消当前的会话。
	 * </p>
	 * <p>
	 * 		在会话被取消后，当前会话结束，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #understandText(String, TextUnderstanderListener)
	 */
	public void cancel()
	{
		if(null != mUnderstander)
		{
			mUnderstander.cancel(false);
		}
		else if(null != mUnderstanderAidl)
		{
			mUnderstanderAidl.cancel(mInternalAidlListener.mAidlListener);
		}
		else
		{
			DebugLog.LogE("TextUnderstander cancel failed, is not running");
		}
	}

	/**
	 * <h4>参数设置</h4>
	 * <p>
	 * 		可设置的参数有：<br>
	 * {@link SpeechConstant#SCENE } ：情景模式 <br>
	 * {@link SpeechConstant#NET_TIMEOUT }： 网络连接超时时间<br>
	 * {@link SpeechConstant#LANGUAGE }：语言<br>
	 * {@link SpeechConstant#ACCENT }：语言区域<br>
	 * {@link SpeechConstant#DOMAIN }：应用领域<br>
	 * {@link SpeechConstant#RESULT_TYPE }：识别结果类型
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型（目前语义仅支持在线模式）；
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return 是否设置成功
	 * 
	 * @see #getParameter(String)
	 * @see #understandText(String, TextUnderstanderListener)
	 */
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}

	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#NET_TIMEOUT}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key
	 *            参数名称，所有支持参数，参考{@link #setParameter(String, String)}
	 * @return 参数值，值意义，参考{@link SpeechConstant}对各参数的说明。
	 * 
	 * @see #setParameter(String, String)
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}

	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createTextUnderstander}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isUnderstanding()}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createTextUnderstander}创建一个新
	 * 的单例对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createTextUnderstander
	 * @see #cancel()
	 */
	public boolean destroy() 
	{
		boolean ret = true;
		
		final TextUnderstanderAidl aidl = this.mUnderstanderAidl;
		if(aidl != null) {
			aidl.destory();
		}
		
		final TextUnderstanderImpl impl = this.mUnderstander;
		if( impl != null ){
			ret = impl.destroy();
		}
		
		if( ret && (ret=super.destroy()) ){
			mUnderstanderAidl = null;
			
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
		}
		
		return ret;
	}
	
	
	
	/**
	 * 内部语义理解监听类，保证回调在主线程完成
	 * 
	 *
	 */
	private final class InternalListener implements TextUnderstanderListener
	{
		private TextUnderstanderListener mOutListener = null;
		private com.iflytek.speech.TextUnderstanderListener mAidlListener = null;
		
		
		private static final int MSG_ERROR = 0;
		private static final int MSG_RESULT = 4;
		// EVENT自定义消息
		private static final int MSG_EVENT  = 6;
		
		public InternalListener(TextUnderstanderListener outListener)
		{
			mOutListener = outListener;
			mAidlListener = new com.iflytek.speech.TextUnderstanderListener.Stub() {
				@Override
				public void onResult(com.iflytek.speech.UnderstanderResult result) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_RESULT, new UnderstanderResult(result.getResultString()));
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onError(int errorCode) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_ERROR, new SpeechError(errorCode));
					mUiHandler.sendMessage(msg);
				}				
			};
		}
		
		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_ERROR:
					mOutListener.onError((SpeechError) msg.obj);
					break;
				case MSG_RESULT:
				{
					mOutListener.onResult((UnderstanderResult)msg.obj);
				}
					break;
				case MSG_EVENT:
//					Message mbody = (Message)msg.obj;
//					// 自定义消息数据
//					mOutListener.onEvent(mbody.what,mbody.arg1,mbody.arg2,(String)mbody.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};
		
		@Override
		public void onResult(UnderstanderResult result) {
			Message msg = mUiHandler.obtainMessage(MSG_RESULT, result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onError(SpeechError error) {
			Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
			mUiHandler.sendMessage(msg);
		}
		
	}
	
}
