package com.iflytek.cloud.msc.ivw.aimic;

import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.msc.ivw.aimic.impl.AIMicImpl;
import com.iflytek.cloud.param.MscKeys;

/**
 * <h3>AIMic类</h3>
 * <p>
 * 		AIMic 类用于录取、处理麦克风的音频，进行唤醒，并把唤醒结果和识别音频返回。
 * 应用通过调用 {@link #startListening()} 开始音频录取、处理，
 * 在开始前，可以通过 {@link #setParameter(String, String)} 设置参数值，
 * 当{@link #KEY_AUDIO_SOURCE}参数值为 {@link #AUDIO_ALSA} 时， AIMic 将开启 ALSA 录音机，
 * 录取音频，并进行处理；当 {@link #KEY_AUDIO_SOURCE}参数值为 {@link #AUDIO_EXTERNAL} 时，
 * 需要应用通过 {@link #writeAudio(byte[], int, int)} 把原始音频传进来。
 * </p>
 * <p>
 * 		通过设置 {@link #KEY_AIMIC_IVW_SSB} 参数，设置唤醒会话的参数，并开启唤醒。唤醒非结果的音频
 * 默认不返回，若需要获取，则需要设置 {@link #KEY_IVW_AUDIO_CB} 。唤醒结果和音频将通过
 * {@link IvwDataListener}返回，应用需要实现该接口，并把实例通过{@link #registerListener(IvwDataListener)}
 * 将其注册到 AIMic，并在不需要再使用时，通过 {@link #unregisterListener(IvwDataListener)} 将其注销。
 * 唤醒结束时，通过设置 {@link #KEY_AIMIC_IVW_SSE} 参数，结束唤醒会话。需要 MSC 的共享文件（libmsc.so）
 * 以及对应的唤醒资源，方可实现此功能。
 * </p>
 * <p>
 * 		在唤醒成功一次后，AIMic 将通过 {@link AsrDataListener} 返回用于识别的音频，直到通过 {@link #reset}
 * 重置 AIMic 的状态或出现错误为止。应用需要实现 {@link AsrDataListener} 接口，并把实例通过
 * {@link #registerListener(AsrDataListener)} 将其注册到 AIMic，并在不需要再使用时，通过 
 * {@link #unregisterListener(AsrDataListener)} 将其注销。
 * </p>
 * <p>
 * 		本例使用单例模式。
 * </p>
 * @author <script>gShowAuthor();</script>
 */
public abstract class AIMic {
	/**
	 * <h4>识别音频缓存时长</h4>
	 * <p>
	 * 		通过此参数设置在唤醒成功时，若没有设置任何 {@link AsrDataListener} 时，最大的识别音频缓存时长。
	 * 当注册到 AIMic 中的 {@link AsrDataListener} 不为空时，将实时的回调识别音频。当此参数值为0，且 AIMc
	 *  中的 {@link AsrDataListener} 为空时，当前识别音频将被抛弃。 因需要使用内存缓存音频，此值不宜设置过
	 *  大，否则将可能由于申请过多内存，而引起OOM问题。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：4000（ms）</p>
	 * <p>值域：[0, 60000]</p>
	 * 
	 * @see #setParameter(String, String)
	 */
	public static final String KEY_ASR_BUFFER_TIME = "aimic_asr_buffer_time";
	
	/**
	 * <h4>音频源</h4>
	 * <p>
	 * 		设置 AIMIC 使用的的音频源。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：{@link #AUDIO_ALSA}</p>
	 * <p>值域：{ {@link #AUDIO_ALSA}, {@link #AUDIO_EXTERNAL} }</p>
	 * 
	 * @see #setParameter(String, String)
	 */
	public static final String KEY_AUDIO_SOURCE = SpeechConstant.AUDIO_SOURCE;
	
	/**
	 * <h4>保存 ALSA 音频</h4>
	 * <p>
	 * 		当{@link #KEY_AUDIO_SOURCE} 为 {@link #AUDIO_ALSA}时，通过此值，可
	 * 决定是否保存 ALSA 录音最近2分钟的音频。当此值为真时，音频将被保存到
	 *  /sdcard/aimic_alsa.pcm 文件中。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：0</p>
	 * <p>值域：{0, 1 } </p>
	 * 
	 * @see #setParameter(String, String)
	 */
	public static final String KEY_SAVE_ALSA = "alsa_save";
	
	/**
	 * <h4>共享库名</h4>
	 * <p>
	 * 		设置 {@link #createAIMic(String)}时加载 AIMic 库的库名，多个库时，以";"分隔。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：aimic;aimicjni </p>
	 * <p>值域：有效的库名字符串 </p>
	 * 
	 * @see #createAIMic(String)
	 */
	public static final String KEY_LIB_NAME = SpeechConstant.LIB_NAME;
	
	/**
	 * <h4>返回唤醒音频</h4>
	 * <p>
	 * 		设置是否在唤醒过程，通过 {@link IvwDataListener#onWakeupAudio(byte[], int, int, Object)} 返回
	 * 所有唤醒音频。此参数只决定唤醒过程的音频数据，唤醒成功的音频数据将不受此参数控制，而通过
	 * {@link IvwDataListener#onWakeupMsg(int, int, int, byte[], int, byte[], int, byte[], int)} 返回。当不需要
	 * 唤醒过程的音频时，建议设置此参数值为假，以减少返回唤醒过程音频时的 CPU 和 内存占用。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：0 </p>
	 * <p>值域：{ 1, 0 } </p>
	 * 
	 * @see #setParameter(String, String)
	 */
	public static final String KEY_IVW_AUDIO_CB = "ivw_audio";
	
	/**
	 * <h4>保存唤醒音频</h4>
	 * <p>
	 * 		设置是否在唤醒结束时，保存最近的唤醒音频。保存的长度通过 {@link #KEY_IVW_BUF_TIME}设置。
	 * 缓存的音频将在唤醒会话结束时，保存到 /sdcard/aimic_ivw_x.pcm 文件中，其中 x 表示第几个唤醒线
	 * 程的音频。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：0 </p>
	 * <p>值域：{ 0, 1 } </p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #KEY_IVW_BUF_TIME 
	 */
	public static final String KEY_IVW_AUDIO_SAVE = "ivw_save_file";
	
	/**
	 * <h4>唤醒音频缓存时长</h4>
	 * <p>
	 * 		设置唤醒过程，缓存最后的唤醒音频的时长，这个值决定在唤醒时，
	 * {@link IvwDataListener#onWakeupMsg(int, int, int, byte[], int, byte[], int, byte[], int)} 能否返回有效的
	 * 唤醒音频。当此值为0时，唤醒时的唤醒音频将也为0。但由于缓存要占用内存，不宜使用过长的缓
	 * 存时长。非必要情况下，建议不需要设置此值，而让SDK使用默认的即可。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：4000（ms）</p>
	 * <p>值域：[ 0, 10000 ]</p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #KEY_IVW_AUDIO_SAVE 
	 */
	public static final String KEY_IVW_BUF_TIME = "ivw_buf_tm";
	
	/**
	 * <h4>唤醒线程数</h4>
	 * <p>
	 * 		设置唤醒过程使用的线程数。AIMic 处理原始的音频，得到 N 个通道的音频，并将此 N 个通道的音频，
	 * 送给不同的唤醒线程进行唤醒。当前仅支持固定的 3 个线程。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：3</p>
	 * <p>值域：{ 3 }</p>
	 * 
	 * @see #setParameter(String, String)
	 */
	public static final String KEY_IVW_THREAD_NUM = "ivw_thread_num";
	
	/**
	 * <h4>唤醒会话开始参数</h4>
	 * <p>
	 * 		设置唤醒会话开始的参数。设置此值并开启唤醒。唤醒会话的具体参数，请参考唤醒类说明。多个参数
	 * 间，以 "," 分隔，如 "ivw_res_path=XXX,ivw_threshold=0:20"。若设置了此参数，则必须在不需要唤醒时，
	 * 通过设置 {@link #KEY_AIMIC_IVW_SSE} 结束唤醒。
	 * </p>
	 * <p>
	 * 		唤醒消息将通过 {@link IvwDataListener#onWakeupMsg(int, int, int, byte[], int, byte[], int, byte[], int)}
	 * 返回。
	 * </p>
	 * 
	 * <p>必须：是（仅在需要开启 AIMic 唤醒时）</p>
	 * <p>默认：null</p>
	 * <p>值域：有效的唤醒参数值</p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #KEY_AIMIC_IVW_SSE
	 * @see IvwDataListener
	 */
	public static final String KEY_AIMIC_IVW_SSB = "ivw_ssb";
	
	/**
	 * <h4>唤醒会话结束参数</h4>
	 * <p>
	 * 		设置唤醒会话结束的参数。设置此值并结束唤醒。在唤醒成功时，值为 "success"，被取消时为 “user abort”，
	 * 失败时为 “error XXX”(XXX 为错误码)。此参数不必在 {@link #startListening()} 前调用，而是在要结束唤醒时调用。
	 * </p>
	 * 
	 * <p>必须：是（仅在需要结束 AIMic 唤醒时）</p>
	 * <p>默认：null</p>
	 * <p>值域：{ "success", "user abort", "error XXX" }</p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #KEY_AIMIC_IVW_SSB
	 */
	public static final String KEY_AIMIC_IVW_SSE = "ivw_sse";
	
	/**
	 * <h4>线程优先级</h4>
	 * <p>必须：否</p>
	 * <p>默认：{@link #PRIORITY_MAX} </p>
	 * <p>值域：{ {@link #PRIORITY_MIN}, {@link #PRIORITY_NORMAL},  {@link #PRIORITY_MAX} }</p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #PRIORITY_MIN
	 * @see PRIORITY_NORMAL
	 * @see PRIORITY_MAX
	 */
	public static final String KEY_THREAD_PRIORITY = "thread_priority";
	
	public static final int PRIORITY_MIN = 0;
	
	public static final int PRIORITY_NORMAL = 1;
	
	public static final int PRIORITY_MAX = 2;
	
	/**
	 * <h4>ALSA录音采样率</h4>
	 * <p>
	 * 		设置 ALSA 录音时的采样率，仅在 {@link #KEY_AUDIO_SOURCE} 值为 {@link #AUDIO_ALSA} 生效。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：参考 ALSA 说明 </p>
	 * <p>值域：参考 ALSA 说明</p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #KEY_AUDIO_SOURCE
	 */
	public static final String KEY_ALSA_SAMPLE_RATE = MscKeys.AIMIC_ALSA_SAMPLE_RATE;
	
	/**
	 * <h4>ALSA录音卡</h4>
	 * <p>
	 * 		设置 ALSA 录音时的卡号，仅在 {@link #KEY_AUDIO_SOURCE} 值为 {@link #AUDIO_ALSA} 生效。
	 * </p>
	 * 
	 * <p>必须：否</p>
	 * <p>默认：参考 ALSA 说明 </p>
	 * <p>值域：参考 ALSA 说明</p>
	 * 
	 * @see #setParameter(String, String)
	 * @see #KEY_AUDIO_SOURCE
	 */
	public static final String KEY_ALSA_CARD = MscKeys.AIMIC_ALSA_CARD;;
	
	/**
	 * <h4>常值值：ALSA音频源</h4>
	 * @ee #KEY_AUDIO_SOURCE 
	 */
	public static final int AUDIO_ALSA = MscKeys.VALUE_AUDIO_AIMIC;
	
	/**
	 * <h4>常值值：外部音频源</h4>
	 * @see #KEY_AUDIO_SOURCE 
	 */
	public static final int AUDIO_EXTERNAL = MscKeys.VALUE_AUDIO_EXTERNAL;
	
	/**
	 * <h3>唤醒数据回调接口</h3>
	 * <p>
	 * 		通过实现此接口，获取 AIMic 返回的唤醒数据，包括唤醒音频和唤醒消息，以及错误码。 
	 * </p>
	 * 
	 * @see AIMic#registerListener(IvwDataListener)
	 * @see AIMic#unregisterListener(IvwDataListener)
	 * @see AsrDataListener
	 */
	public static interface IvwDataListener{
		/**
		 * <h4>唤醒过程音频</h4>
		 * <p>
		 * 		返回唤醒过程的音频，不管唤醒是否有开启，或唤醒是否成功。唤醒音频在
		 *  {@link AIMic#KEY_AIMIC_IVW_SSB} 参数为真时，音频在 aimicjni 共享库中自
		 *  动已写入的唤醒线程，不需要应用再把此函数返回的音频，写入到唤醒接口中。
		 * </p>
		 * <p>
		 * 		返回的唤醒音频，是多个通道的音频，通道数取决于{@link AIMic#KEY_IVW_THREAD_NUM}，
		 * 如该参数值为 3，且返回的是 1536 字节，则每个通道的音频字节数为 1536 / 3 = 512，前 512 字节
		 * 为第一个通道的音频，中间的 512 字节为第二个通道的音频，最后的 512 字节为第三个通道的音频。
		 * </p>
		 * <p>
		 * 		仅接口仅在 {@link AIMic#KEY_IVW_AUDIO_CB} 参数值为真时回调。
		 * </p>
		 * @param audioData 音频数据
		 * @param dataLen 数据长度
		 * @param arg1 保留
		 * @param arg2 保留
		 */
		public void onWakeupAudio(byte[] audioData, int dataLen, int arg1, Object arg2);

		/**
		 * <h4>唤醒消息</h4>
		 * <p>
		 * 		返回唤醒消息，仅在已开启唤醒，且有唤醒消息时，回调此函数。
		 * </p>
		 * @param msg 消息 id ， 1：唤醒成功；2：唤醒错误；3，识别结果；4：发现识别音频端点； 
		 * 		5：音量；6：注册消息；7：唤醒被重置； 其中，3, 4 仅在唤醒识别时出现，5 暂不支持，
		 * 6 仅在唤醒注册时出现，暂不支持。
		 * @param arg1 辅助参数1，在 msg=2 时，此值表示具体本的错误码；msg=4 时，此值表示端
		 * 点状态；msg=5时，此值表示音量值；
		 * @param arg2 保留
		 * @param sid 会话ID
		 * @param sidLen 会话ID字符串长度
		 * @param result 唤醒/识别结果
		 * @param resultLen 唤醒/识别结果长度 
		 * @param audio 唤醒音频
		 * @param audioLen 唤醒音频长度
		 * 
		 * @see AIMic#KEY_AIMIC_IVW_SSB
		 */
		public void onWakeupMsg(int msg, int arg1, int arg2, byte[] sid, int sidLen, byte[] result, int resultLen, byte[] audio, int audioLen );
		
		/**
		 * <h4>错误消息</h4>
		 * <p>
		 * 		当出现错误时，通过此接口返回错误码。具体错误码值定义，参考 {@link com.iflytek.cloud.ErrorCode}。
		 * </p>
		 * @param errorCode 错误码
		 * 
		 * @see com.iflytek.cloud.ErrorCode
		 */
		public void onError( int errorCode );
	}
	
	/**
	 * <h3>识别数据回调接口</h3>
	 * <p>
	 * 		通过实现此接口，获取 AIMic 返回的识别数据，包括识别音频和错误码。 
	 * </p>
	 * 
	 * @see AIMic#registerListener(AsrDataListener)
	 * @see AIMic#unregisterListener(AsrDataListener)
	 * @see IvwDataListener
	 */
	public static interface AsrDataListener{
		/**
		 * <h4>返回识别音频</h4>
		 * <p>
		 * 		在一次成功唤醒后，调用 {@link AIMic#reset()} 为止，AIMic 会一直通过此函数，返回可用于识别
		 * 的音频。 应用收到此函数返回的音频后，可将其传给识别接口，用于识别。
		 * </p>
		 * <p>
		 * 		当前返回的识别音频格式为，未压缩的16位，单声道，采样率为16000，字节顺序为
		 * Little-Endian 的Windows PCM音频。
		 * </p>
		 * @param audioData 音频数据缓存
		 * @param dataLen 有效数据长度
		 * @param arg1 保留
		 * @param arg2 保留
		 */
		public void onRecogAudio(byte[] audioData, int dataLen, int arg1, Object arg2);
		
		/**
		 * <h4>错误消息</h4>
		 * <p>
		 * 		当出现错误时，通过此接口返回错误码。具体错误码值定义，参考 {@link com.iflytek.cloud.ErrorCode}。
		 * </p>
		 * @param errorCode 错误码
		 * 
		 * @see com.iflytek.cloud.ErrorCode
		 */
		public void onError( int errorCode );
	}

	/**
	 * <h4>获取 AIMic 实例</h4>
	 * <p>
	 * 		获取已存在的 AIMic 实例。若当前没有实例存在，将返回 null。
	 * </p>
	 * @return AIMic 实例，或 null
	 * 
	 * @see #createAIMic(String)
	 */
	public static AIMic getAIMic(){
		return AIMicImpl.getAIMicImpl();
	}
	
	/**
	 * <h4>创建 AIMic 实例</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用 {@link #destroy} 销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy} 前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getAIMic()} 获取已创建的单例对象。
	 * </p>
	 * @param params
	 * @return AIMic 对象 
	 * 
	 * @see #getAIMic()
	 * @see #destroy()
	 */
	public static AIMic createAIMic( final String params ){
		return AIMicImpl.createAIMicImpl( params );
	}
	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createAIMic}创建的单例对象。此时，之前创建的单
	 * 例对象已不能再使用，否则，将会报错。此时需要再使用，应先通过
	 * {@link #createAIMic}创建一个新的单例对象。
	 * </p>
	 * 
	 * @see #createAIMic
	 */
	public abstract void destroy();
	
	/**
	 * <h4>注册监听器</h4>
	 * <p>
	 * 		注册监听器，以接收SDK通过监听器返回的数据。不再使用时，必须通过 
	 * {@link  #unregisterListener(IvwDataListener)} 注销。 
	 * </p>
	 * @param listener
	 * 
	 * @see #unregisterListener(IvwDataListener)
	 */
	public abstract void registerListener( final IvwDataListener listener );
	
	/**
	 * <h4>注册监听器</h4>
	 * <p>
	 * 		注册监听器，以接收SDK通过监听器返回的数据。不再使用时，必须通过 
	 * {@link  #unregisterListener(AsrDataListener)} 注销。 
	 * </p>
	 * @param listener
	 * 
	 * @see #unregisterListener(AsrDataListener)
	 */
	public abstract void registerListener( final AsrDataListener listener );
	
	/**
	 * <h4>注销监听器</h4>
	 * <p>
	 * 		注销监听器后， AIMic 将不再持有监听器的引用，监听器对象将不再
	 * 收到 AIMic 的回调。
	 * </p>
	 * @param listener
	 * 
	 * @see #registerListener(IvwDataListener)
	 */
	public abstract void unregisterListener( final IvwDataListener listener );
	
	/**
	 * <h4>注销监听器</h4>
	 * <p>
	 * 		注销监听器后， AIMic 将不再持有监听器的引用，监听器对象将不再
	 * 收到 AIMic 的回调。注销监听器应在调用 {@link #stopListening() }前。
	 * </p>
	 * @param listener
	 * 
	 * @see #registerListener(AsrDataListener)
	 */
	public abstract void unregisterListener( final AsrDataListener listener );
	
	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		设置参数，具体支持参数参考本类 String 类型的成员说明。
	 * </p>
	 * @param key 参数名
	 * @param value 参数值，所有其他类型均转为 String 类型传入
	 * @return 状态， {@link com.iflytek.cloud.ErrorCode#SUCCESS} 时为成功，否则为失败。具体错误码，参考
	 * 		{@link com.iflytek.cloud.ErrorCode}
	 * 
	 * @see #startListening()
	 * @see #getParameter(String)
	 */
	public abstract int setParameter( final String key, final String value );
	
	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取参数值。
	 * </p>
	 * @param key 参数名
	 * @return 参数值
	 * 
	 * @see #setParameter(String, String)
	 */
	public abstract String getParameter( final String key );
	
	/**
	 * <h4>重置 AIMic</h4>
	 * <p>
	 * 		重置 AIMic 状态，但不包括 AIMic 中唤醒线程的状态。重置 AIMic 后，直到下次成功唤醒
	 * 为止，将不再返回识别音频。
	 * </p>
	 * 
	 * @return 状态， {@link com.iflytek.cloud.ErrorCode#SUCCESS} 时为成功，否则为失败。具体错误码，参考
	 * 		{@link com.iflytek.cloud.ErrorCode}
	 * 
	 * @see #startListening()
	 * @see AsrDataListener
	 */
	public abstract int reset();
	
	/**
	 * <h4>开始监听</h4>
	 * <p>
	 * 		开始录音（仅当 {@link #KEY_AUDIO_SOURCE} 值为{@link #AUDIO_ALSA}时），并处理音频。
	 * AIMic 使用单例模式，当监听开始后，再次调用开始监听时，将保持原来的监听，不进行任何其
	 * 他操作。若应用已知当前 AIMic 处于监听状态中，可直接通过 {@link #registerListener(AsrDataListener)},
	 *  {@link #registerListener(IvwDataListener)}把自己的监听器注册到 AIMic ，即可收到 AIMic 返回的
	 *  数据。 
	 * </p>
	 * <p>
	 * 		当{@link #KEY_AUDIO_SOURCE} 为 {@link #AUDIO_EXTERNAL}时，应用需要通过
	 *  {@link #writeAudio(byte[], int, int)} 把音频数据传入。
	 * </p>
	 * 
	 * @return 状态， {@link com.iflytek.cloud.ErrorCode#SUCCESS} 时为成功，否则为失败。具体错误码，参考
	 * 		{@link com.iflytek.cloud.ErrorCode}
	 * 
	 * @see #isListening()
	 * @see #stopListening()
	 * @see #registerListener(AsrDataListener)
	 * @see #registerListener(IvwDataListener)
	 * @see #writeAudio(byte[], int, int)
	 */
	public abstract int startListening();
	
	/**
	 * <h4>结束监听</h4>
	 * <p>
	 * 		调用停止监听后， AIMic 将根据当前是否还有 {@link IvwDataListener}, {@link AsrDataListener} 
	 * 监听器，如果有，则不会停止监听，否则停止监听。停止监听时，若{@link #KEY_AUDIO_SOURCE}
	 * 值为 {@link #AUDIO_ALSA}， 将停止录音。
	 * </p>
	 * 
	 * <p>
	 * 		结束监听不会自动结束唤醒会话，即如果已设置了 {@link #KEY_AIMIC_IVW_SSB}，必须再次设置
	 * {@link #KEY_AIMIC_IVW_SSE} 以结束唤醒会话。
	 * </p>
	 * 
	 * @see #startListening()
	 * @see #isListening()
	 * @see #unregisterListener(AsrDataListener)
	 * @see #unregisterListener(IvwDataListener)
	 */
	public abstract void stopListening();
	
	/**
	 * <h4>是否在监听中</h4>
	 * <p>
	 * 		判断当前是否在监听中。
	 * </p>
	 * @return 布尔型值
	 * 
	 * @see #startListening()
	 * @see #stopListening()
	 */
	public abstract boolean isListening();
	
	/**
	 * <h4>写入音频</h4>
	 * <p>
	 * 		仅在 {@link #KEY_AUDIO_SOURCE} 值为 {@link #AUDIO_EXTERNAL}时，通过此函数
	 * 写入音频数据。写入音频数据的大小和间隔，应与实际录音得到的音频大小间隔相似，过快
	 * 写入音频，将可能使SDK由于缓存不够而丢失音频，或报错。
	 * </p>
	 * <p>
	 * 		当前支持的音频为使用 ALSA 当制的原始音频，具体参考 ALSA 相关文档说明。
	 * </p>
	 * 
	 * @param audioData 音频数据
	 * @param offset 有效数据开始偏移大小
	 * @param dataLen 有效数据长度
	 * @return 状态， {@link com.iflytek.cloud.ErrorCode#SUCCESS} 时为成功，否则为失败。具体错误码，参考
	 * 		{@link com.iflytek.cloud.ErrorCode}
	 * 
	 * @see #KEY_AUDIO_SOURCE
	 * @see #startListening()
	 */
	public abstract int writeAudio( final byte[] audioData, final int offset, final int dataLen );
	
	/**
	 * <h4>获取版本号</h4>
	 * 
	 * @return 版本号
	 */
	public static String getVesion(){
		return AIMicImpl.getVesion();
	}
	
	/**
	 * <h4>设置日志打印</h4>
	 * <p>
	 * 		默认下会进行日志打印，若不需要，则设置为 false。
	 * </p>
	 * @param enable 是否启用日志打印
	 */
	public abstract void setShowLog( final boolean enable, final int level );
	
	public static int getChannel(){
		return AIMicImpl.getChannel();
	}
}
