/*
 * 文件名：SpeechSynthesizerImpl.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：语音合成对象实现；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.speech.impl;

import android.content.Context;
import android.text.TextUtils;
import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SynthesizerListener;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.tts.MscSynthesizer;
import com.iflytek.cloud.msc.tts.SpeakSession;
import com.iflytek.cloud.msc.tts.TtsSession;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.record.PcmPlayer;

public class SpeechSynthesizerImpl extends SpeechModule implements SpeakSession.BufferReadyListener{

	private Context mContext = null;
	
	// 当前会话
	private SpeakSession  mCurSession = null;
	// 缓冲会话
	private SpeakSession mNextSession = null;

	/**
	 * 合成对象初始化
	 * @param context 上下文环境
	 */
	public SpeechSynthesizerImpl(Context context) {
		super(context);
		mContext = context.getApplicationContext();
	}
	
	/**
	 * 开始合成
	 * @param text 合成文本
	 * @param listener 合成回调监听器
	 */
	public int startSpeaking(String text, SynthesizerListener listener) 
	{
		DebugLog.LogD( "startSpeaking enter" );
		int ret = ErrorCode.SUCCESS;
		
		synchronized( this ){
			final String nextText = mSessionParams.remove( SpeechConstant.NEXT_TEXT );
			// 记录用户点击时间
			if(null!=mCurSession && mCurSession.isSpeaking())
			{
				// 判断是否通知外部合成被异常打断
				mCurSession.cancel(mSessionParams.getBoolean(SpeechConstant.TTS_INTERRUPT_ERROR, false));
			}
			
			if (null == mNextSession)
			{
				//上一次未进行预合成
				ret = startSession(text, listener, nextText);
			}
			else if (!text.equals(mNextSession.mText)) 
			{
				//上一次进行了预合成 且 本次文本与上一次预合成的文本不一致
				//取消上一次预合成，开始本次合成。
				mNextSession.cancel(false);
				mNextSession = null;
				
				ret = startSession(text, listener, nextText);
			}
			else 
			{
				//上一次进行了预合成
				if (mNextSession.mError != null || !mNextSession.synthesizeBegin) 
				{
					//上一次预合成出错了，开始本次合成。   ||  上次进行了预合成，但还没开始（多线程快速开启下一次时会出现）,开始本次合成
					mNextSession.cancel(false);
					mNextSession = null;
					ret = startSession(text, listener, nextText);
				}
				else 
				{
					//使用上一次预合成的进行播报
					SpeakSession temp = mNextSession;
					mNextSession = null;
					if( !TextUtils.isEmpty(nextText) ){
						mNextSession = new SpeakSession(mContext);
						mNextSession.setBufferReadyLietener(this);
						mNextSession.setBufferText(nextText, mSessionParams);
					}

					mCurSession = temp;
					mCurSession.setListener(listener);
					mCurSession.resumeSpeaking();

					if(mCurSession.bufferComplete)
					{
						this.onBufferReady();
						DebugLog.LogD("startSpeaking NextSession pause");
					}
				}
			}
		}//end of synchronized
		
		DebugLog.LogD( "startSpeaking leave" );
		return ret;
	}

	private int startSession(String text, SynthesizerListener listener,
			final String nextText) {
		int ret = ErrorCode.SUCCESS;
		
		DebugLog.LogD("new Session Start");
		mCurSession = new SpeakSession(mContext);
		mCurSession.setBufferReadyLietener(this);
		final String local_save_path = mSessionParams.getString(SpeechConstant.TTS_AUDIO_PATH);
		ret = mCurSession.startSpeaking(text, mSessionParams, listener,true, local_save_path);

		if( !TextUtils.isEmpty(nextText) ){
			mNextSession = new SpeakSession( mContext);
			mNextSession.setBufferReadyLietener(this);
			mNextSession.setBufferText(nextText, mSessionParams);
		}
		
		return ret;
	}
	
	/**
	 * 合成完成接口，表示可以开发预先合成下一路会话
	 */
	@Override
	public void onBufferReady() {
		synchronized( this ){
			if(mNextSession != null)
			{
				mNextSession.startBufferText();
			}
		}	
	}
	
	/**
	 * 开始合成
	 * @param text 合成文本
	 * @param listener 合成回调监听器
	 */
	public int synthesizeToUri(String text, String uri, SynthesizerListener listener) {
		DebugLog.LogD( "synthesizeToUri enter" );
		int ret = ErrorCode.SUCCESS;
		
		synchronized( this ){
			// 再次启动需要判断是否存在正在运行的合成会话
			if(mCurSession != null && mCurSession.isSpeaking())
			{
				// 判断是否通知外部合成被异常打断
				mCurSession.cancel(mSessionParams.getBoolean(SpeechConstant.TTS_INTERRUPT_ERROR, false));
			}
			mCurSession = new SpeakSession(mContext);
			ret = mCurSession.synthesizeToUri(text,uri, mSessionParams, listener);
		}//end of synchronized
		
		DebugLog.LogD( "synthesizeToUri leave" );
		return ret;
	}
	
	/**
	 * 暂停合成
	 */
	public void pauseSpeaking() {
		DebugLog.LogD( "pauseSpeaking enter" );
		synchronized( this ){
			if(mCurSession != null)
			{
				// 暂停事件
				mCurSession.pauseSpeaking();
			}
		}
		DebugLog.LogD( "pauseSpeaking leave" );
	}
	/**
	 * 重播
	 */
	public void replaySpeaking() 
	{
		DebugLog.LogD( "replaySpeaking enter" );
		synchronized( this ){
			if(mCurSession != null)
			{
				// 重播事件
				mCurSession.replaySpeaking();
			}
		}
		DebugLog.LogD( "replaySpeaking leave" );
	}

	/**
	 * 恢复播放.
	 */
	public void resumeSpeaking() 
	{
		DebugLog.LogD( "resumeSpeaking enter" );
		synchronized( this ){
			if(mCurSession != null)
			{
				// 恢复播放
				mCurSession.resumeSpeaking();
			}
		}
		DebugLog.LogD( "resumeSpeaking leave" );
	}
	
	/**
	 * 根据当前会话是否活动，判断新的会话是否可以开启
	 * @return
	 */
	public boolean isSpeaking() 
	{
		boolean ret = false;
		DebugLog.LogD( "isSpeaking enter" );
		synchronized( this ){
			if(mCurSession != null)
			{
				// 是否正在播放
				ret = mCurSession.isSpeaking();
			}
		}
		DebugLog.LogD( "isSpeaking leave" );
		
		return ret;
	}
	
	/**
	 * 当前合成状态
	 */
	public int getState() {
		int ret = PcmPlayer.STOPED;
		DebugLog.LogD( "getState enter" );
		synchronized( this ){
			if(mCurSession != null)
				ret = mCurSession.getState();
		}
		DebugLog.LogD( "getState leave" );
		
		return ret;
	}
	
	/**
	 * 取消事件，需要保存消息，有可能会话已经结束，仍需要保存
	 */
	public void stopSpeaking(boolean notifyError) {
		DebugLog.LogD( "stopSpeaking enter:"+notifyError );
		synchronized( this ){
			if(mCurSession != null){
				DebugLog.LogD("-->stopSpeaking cur");
				mCurSession.cancel(notifyError);
				mCurSession = null;
			}
			
			if( null != mNextSession ){
				DebugLog.LogD("-->stopSpeaking cur next");
				mNextSession.cancel( false );
				mNextSession = null;
			}
		}
		DebugLog.LogD( "stopSpeaking leave" );
	}

	
	/**
	 * 获取下载流量
	 */
	public int getDownflowBytes(boolean isTotal) {
		if(isTotal)
			return TtsSession.getIntByTag(MscHandler.TAG_DOWNFLOW);
		else
			return MscSynthesizer.mDownflow;
	}

	/**
	 * 获取上传流量
	 */
	public int getUpflowBytes(boolean isTotal) {
		if(isTotal)
			return TtsSession.getIntByTag(MscHandler.TAG_UPFLOW);
		else
			return MscSynthesizer.mUpflow;
	}
	/**
	 * 销毁合成对象
	 */
	public boolean destroy() {
		stopSpeaking(false);
		super.destroy();
		return true;
	}
}
