package com.iflytek.cloud.util.contacts;

import java.util.regex.Pattern;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.util.contacts.entities.ContactSpecNumber;

/**
 * 手机号码相关的工具类
 * @author weiwei
 * @date 2012-11-14
 */
public class PhoneNumberUtil {
	
	/**
	 * 移动：[3G标准：TD-SCDMA]
	 * 共有16个手机号段，其中147、157、188是3G专用号段，其它为2G号段。
	 * 目前147开头的号码主要用于上网本等数据业务，157则用于G3无线固话，188用于3G手机
	 */
	public static final String[] MobileNumberHead = {"134", "135", "136", "137", "138", "139", 
													 "147", "150", "151", "152", "157", "158",
													 "159", "182", "187", "188"};
	
	/**
	 * 联通：[3G标准：WCDMA]
	 * 共7种号段，其中186是3G专用号段，其余为2G号段
	 */
	public static final String[] UnicomNumberHead = {"130", "131", "132", "155", "156", "185", "186"};
	
	/**
	 * 电信：[3G标准：CDMA2000]
	 * 共4种，接手原联通CDMA网络的133、153号段，189采用CDMA20003G标准
	 */
	public static final String[] TelecomNumberHead = {"133", "153", "180", "189"};
	
	public static final int MOBILE_NUMBER_SIZE = 11;
	/**
	 * 移动业务运营商
	 */
	public enum MobileServiceProvider {
		/** 移动 */
		mobile,
		/** 联通 */
		unicom,
		/** 电信 */ 
		telecom,
		/** 其它 */
		other
	}
	
	/**
	 * 是否移动号码
	 * @param number
	 * @return
	 */
	public static boolean isChinaMobile(String number) {
	    if (null == number) {
	        return false;
	    }
		if (number.trim().length() > 0 && isMobileNumber(number)) {
			for (int index = 0; index < MobileNumberHead.length; index++) {
				if (number.startsWith(MobileNumberHead[index])) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * 是否联通号码
	 * @param number
	 * @return
	 */
	public static boolean isChinaUnicom(String number) {
	    if (null == number) {
            return false;
        }
		if (number.trim().length() > 0 && isMobileNumber(number)) {
			for (int index = 0; index < UnicomNumberHead.length; index++) {
				if (number.startsWith(UnicomNumberHead[index])) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 是否电信号码
	 * @param number
	 * @return
	 */
	public static boolean isChinaTelecom(String number) {
	    if (null == number) {
            return false;
        }
		if (number.trim().length() > 0 && isMobileNumber(number)) {
			for (int index = 0; index < TelecomNumberHead.length; index++) {
				if (number.startsWith(TelecomNumberHead[index])) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
     * 判断字符串是否为数字
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        try {
            Pattern pattern = Pattern.compile("[0-9]*");
            return pattern.matcher(str).matches();
        } catch (Exception e) {
            DebugLog.LogE(e);
            return true;
        }
    }
	
	/**
     * 过滤号码中的特殊字符
     */
    public static String filterHeadNumber(String number) {
        StringBuilder selectBuilder = null; 
        String selectNum = "";
        if (null != number) {
            number = number.replaceAll(" ", "");
            number = number.replaceAll("-", "");
            selectBuilder = new StringBuilder(number);
            
            if (selectBuilder.length() > 5) {
                if (selectBuilder.substring(0, 3).equals(ContactSpecNumber.SPEC_NUM_PLUS_86) ||
                        selectBuilder.substring(0, 3).equals(ContactSpecNumber.SPEC_NUM_086)) {
                    selectNum = selectBuilder.substring(3, selectBuilder.length());
                    return selectNum;
                }
                if (selectBuilder.substring(0, 2).equals(ContactSpecNumber.SPEC_NUM_86)) {
                    selectNum = selectBuilder.substring(2, selectBuilder.length());
                    return selectNum;
                }
                if (selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_12530) ||
                        selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_12520) ||
                        selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_17951) ||
                        selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_17911) ||
                        selectBuilder.subSequence(0, 5).equals(ContactSpecNumber.SPEC_NUM_12593)) {
                    selectNum = selectBuilder.substring(5, selectBuilder.length());
                    return selectNum;
                }
            }
        }
        return number;
    }
    
    /**
     * 过滤号码中的空格
     * @param number
     * @return
     */
    public static String filterEmptyNumber(String number) {
        if (null == number) {
            return null;
        }
        number = number.replaceAll(" ", "");
        number = number.replaceAll("-", "");
        return number;
    }
    
    /**
     * 判断当前号码是否包含特服前缀，包括17951、12530、12520、17911、+86、86、086
     * @param number
     * @return 如果包含前缀，返回前缀，如果不包含，返回null
     */
    public static String hasSpecNumber(String number) {
        if (null == number) {
            return null;
        }
        number = number.replaceAll(" ", "");
        number = number.replaceAll("-", "");
        StringBuilder selectBuilder = new StringBuilder(number);
        // 17951|17911|12530|12520并且是手机的情况
        if (selectBuilder.length() == 16) {
            if (selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_12530) ||
                    selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_12520) ||
                    selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_17951) ||
                    selectBuilder.substring(0, 5).equals(ContactSpecNumber.SPEC_NUM_17911) ||
                    selectBuilder.subSequence(0, 5).equals(ContactSpecNumber.SPEC_NUM_12593)) {
                return selectBuilder.substring(0, 5).toString();
            }
        }
        // 0086并且是手机的情况
        else if (selectBuilder.length() == 15) {
            if (selectBuilder.substring(0, 4).equals(ContactSpecNumber.SPEC_INTERNACTIONAL_86)) {
                return selectBuilder.substring(0, 4).toString();
            }
        }
        // +86|086并且是手机的情况
        else if (selectBuilder.length() == 14) {
            if (selectBuilder.substring(0, 3).equals(ContactSpecNumber.SPEC_NUM_086) ||
                    selectBuilder.substring(0, 3).equals(ContactSpecNumber.SPEC_NUM_PLUS_86)) {
                return selectBuilder.substring(0, 3).toString();
            }
        }
        // 86并且是手机的情况
        else if (selectBuilder.length() == 13) {
            if (selectBuilder.substring(0, 2).equals(ContactSpecNumber.SPEC_NUM_86)) {
                return selectBuilder.substring(0, 2).toString();
            }
        }
        return null;
    }
    
    
    /**
     * 判断是否为手机号码
     * @param number
     * @return true 是手机号码   false 是固定电话
     */
    public static boolean isMobileNumber(String number) {
        if (null == number || number.length() <= 3) {
            return false;
        }
        
        StringBuilder numberBuilder = new StringBuilder(number);
        // 如果前2位是00开头，认为是海外号码，属于手机号码类别
        if (numberBuilder.length() > 2 && numberBuilder.toString().startsWith("00")) {
            return true;
        }

        if (numberBuilder.length() == 11) {
            if (numberBuilder.toString().startsWith("1")) {
                return true;
            }
        } else if (numberBuilder.length() == 16) {
            String tmp = numberBuilder.substring(0, 5);
            if (tmp.equals(ContactSpecNumber.SPEC_NUM_12520) || tmp.equals(ContactSpecNumber.SPEC_NUM_12530) ||
                    tmp.equals(ContactSpecNumber.SPEC_NUM_17951) || tmp.equals(ContactSpecNumber.SPEC_NUM_17911) ||
                    tmp.equals(ContactSpecNumber.SPEC_NUM_12593)) {
                Character remain = number.charAt(5);
                if (remain.toString().equals("1")) {
                    return true;
                }
            } 
        } else if (numberBuilder.length() == 14) {
             String tmp = numberBuilder.substring(0, 3);
             if (tmp.equals(ContactSpecNumber.SPEC_NUM_086) || tmp.equals(ContactSpecNumber.SPEC_NUM_PLUS_86)) {
                 Character remain = number.charAt(3);
                 if (remain.toString().equals("1")) {
                     return true;
                 }
             } 
        } else if (numberBuilder.length() == 13) {
            String tmp = numberBuilder.substring(0, 2);
            if (tmp.equals(ContactSpecNumber.SPEC_NUM_86)) {
                Character remain = number.charAt(2);
                if (remain.toString().equals("1")) {
                    return true;
                }
            } 
        }
        
        return false;
    }
    
    /**
     * 过滤固定电话号码的区号
     * @param number
     * @return 过滤后的号码
     */
    public static String filterFixedNumber(String number) {
        if (number.startsWith("0")) {
            if (number.startsWith("01") || number.startsWith("02")) {    //一般固定电话的区号开头是01或02的是3位
                return number.substring(3);    //去掉前3位
            } else {
                return number.substring(4);    //去掉前4位
            }
        }
        return number;
    }
    
    /**
     * 数据库模糊查询使用，将字符串中所有空格替换为_
     * @param name
     * @return 替换后的字符串
     */
    public static String filterSqlName(String name) {
        if (null == name) {
            return null;
        }
        if (name.contains(" ")) {
            name = name.replaceAll(" ", "_");
        }
        return name;
    }
    
    /**
     * 合并字符数组为字符串，以特定字符ch为分隔
     * @param items
     * @param ch
     * @return
     */
    public static String mergeStrings(String[] items, char ch) {
        StringBuilder buf = new StringBuilder();
    	if (null != items) {
            for (String s : items) {
                if (s != null) {
                    buf.append(s);
                    buf.append(ch);
                }
            }
        }
        return buf.toString();
    }
	
}
